/*********************************************************************
*
*   MODULE NAME:
*       CCdtPingDlg.cpp
*
* Copyright 2014 by Garmin Ltd. or its subsidiaries.
*---------------------------------------------------------------------
* $NoKeywords$
*********************************************************************/
#include "stdafx.h"
#include "CCdtPingDlg.h"

#if( CDT_SUPPORT )

// CCdtPingDlg dialog

IMPLEMENT_DYNAMIC(CCdtPingDlg, CDialog)

enum TimerIdType
{
    PING_TIMER_ID = 1
};

BEGIN_MESSAGE_MAP(CCdtPingDlg, CDialog)
    ON_WM_SIZE()
    ON_MESSAGE( WM_EVENT( EVENT_CDT_PING_RECEIVED ), OnCDTPingReceived )
    ON_BN_CLICKED(IDC_TOGGLE_PROCESS, &CCdtPingDlg::OnBnClickedToggleProcess)
    ON_WM_TIMER()
    ON_BN_CLICKED(IDC_CLEAR_RESULTS, &CCdtPingDlg::OnBnClickedClearResults)
END_MESSAGE_MAP()

//----------------------------------------------------------------------
//! \brief Constructor
//----------------------------------------------------------------------
CCdtPingDlg::CCdtPingDlg(CWnd* pParent, FmiApplicationLayer & aCom)
    : CDialog( CCdtPingDlg::IDD, pParent ),
    mCom( aCom )
    , mTimer( 0 )
    , mInterval( 1000 )
    , mPayloadSize( 0 )
    , mRepeatIterations( 1 )
    , mRepeatFixed( FALSE )
    , mClearOnStart( TRUE )
    {
    }

//----------------------------------------------------------------------
//! \brief Destructor
//----------------------------------------------------------------------
CCdtPingDlg::~CCdtPingDlg()
{
    for( int i=0; i<mResults.GetCount(); i++ )
        {
        delete mResults[i];
        }
    mResults.RemoveAll();
}

//----------------------------------------------------------------------
//! \brief Perform dialog data exchange and validation
//! \param  aDataExchange The DDX context
//----------------------------------------------------------------------
void CCdtPingDlg::DoDataExchange(CDataExchange* pDX)
   {
   CDialog::DoDataExchange(pDX);
   DDX_Text(pDX, IDC_INTERVAL, mInterval);
   DDV_MinMaxUInt(pDX, mInterval, 100, 5000);
   DDX_Text(pDX, IDC_PAYLOAD_SIZE, mPayloadSize);
   DDV_MinMaxUInt(pDX, mPayloadSize, 0, 246);
   DDX_Text(pDX, IDC_REPEAT_ITERATIONS, mRepeatIterations);
   DDV_MinMaxUInt(pDX, mRepeatIterations, 1, 1000);
   DDX_Radio(pDX, IDC_REPEAT_INDEFINITELY, mRepeatFixed);
   DDX_Control(pDX, IDC_PING_RESULTS, mResultsView);
   DDX_Check(pDX, IDC_CLEAR_ON_START, mClearOnStart);
    }

//----------------------------------------------------------------------
//! \brief This function is called when the window is created.
//! \return TRUE, since this function does not set focus to a control
//----------------------------------------------------------------------
BOOL CCdtPingDlg::OnInitDialog()
    {
    CDialog::OnInitDialog();

    mResultsView.InsertColumn( 0, _T( "Sent On" ), LVCFMT_LEFT, 120);
    mResultsView.InsertColumn( 1, _T( "Responded On" ), LVCFMT_RIGHT, 120 );
    mResultsView.InsertColumn( 2, _T( "Elapsed Time" ), LVCFMT_RIGHT, 100 );
    mResultsView.InsertColumn( 3, _T( "Match" ), LVCFMT_CENTER, 50 );

    (( CSpinButtonCtrl*) GetDlgItem( IDC_REPEAT_ITERATIONS_SPINNER ) )->SetRange( 1, 1000 );
    (( CSpinButtonCtrl*) GetDlgItem( IDC_INTERVAL_SPINNER ) )->SetRange( 100, 5000 );
    (( CSpinButtonCtrl*) GetDlgItem( IDC_PAYLOAD_SIZE_SPINNER ) )->SetRange( 0, 246 );

    OnBnClickedClearResults();

    return TRUE;
    }

//----------------------------------------------------------------------
//! \brief Clear and cleanup results
//----------------------------------------------------------------------
void CCdtPingDlg::OnBnClickedClearResults()
    {
    for( int i=0; i<mResults.GetCount(); i++ )
        {
        delete mResults[i];
        }
    mResults.RemoveAll();

    mResultsView.DeleteAllItems();
    GetDlgItem( IDC_TOTAL_DELIVERED_STATIC )->SetWindowText( _T( "" ) );
    GetDlgItem( IDC_AVG_ELAPSED_TIME_STATIC )->SetWindowText( _T( "" ) );
    GetDlgItem( IDC_TOTAL_MISMATCHED_STATIC )->SetWindowText( _T( "" ) );
    }

//----------------------------------------------------------------------
//! \brief Start or Stop sending CDT Pings
//----------------------------------------------------------------------
void CCdtPingDlg::OnBnClickedToggleProcess()
    {
    if( mTimer == 0 )
        {
        if( !UpdateData() )
            {
            return;
            }
        if( mClearOnStart )
            {
            OnBnClickedClearResults();
            }
        mIteration = 0;
        mTimer = SetTimer( PING_TIMER_ID, mInterval, NULL );
        OnTimer( PING_TIMER_ID );
        }
    else
        {
        KillTimer( mTimer );
        mTimer = 0;
        }

    setStarted( mTimer != 0 );
    }

//----------------------------------------------------------------------
//! \brief Update display based on processing state
//----------------------------------------------------------------------
void CCdtPingDlg::setStarted( BOOL started )
    {
    GetDlgItem( IDC_TOGGLE_PROCESS )->SetWindowText( started ? _T( "Stop" ) : _T( "Start" ) );
    GetDlgItem( IDC_REPEAT_INDEFINITELY )->EnableWindow( !started );
    GetDlgItem( IDC_REPEAT_FIXED )->EnableWindow( !started );
    GetDlgItem( IDC_REPEAT_ITERATIONS )->EnableWindow( !started );
    GetDlgItem( IDC_REPEAT_ITERATIONS_SPINNER )->EnableWindow( !started );
    GetDlgItem( IDC_INTERVAL )->EnableWindow( !started );
    GetDlgItem( IDC_INTERVAL_SPINNER )->EnableWindow( !started );
    GetDlgItem( IDC_PAYLOAD_SIZE )->EnableWindow( !started );
    GetDlgItem( IDC_PAYLOAD_SIZE_SPINNER )->EnableWindow( !started );
    }

//----------------------------------------------------------------------
//! \brief Handle timer events, specifically for sending new CDT Pings
//----------------------------------------------------------------------
void CCdtPingDlg::OnTimer
    (
    UINT aTimerId
    )
{
    switch ( aTimerId )
        {
        case PING_TIMER_ID:
            {
            if( !mRepeatFixed || mIteration < mRepeatIterations )
                {
                COleDateTime now = COleDateTime::GetCurrentTime();
                CdtPingTestPacket * packet = new CdtPingTestPacket( mPayloadSize );
                mResults.Add( packet );
                mCom.txCdt( CDT_PING_REQUEST, packet->GetPayload(), packet->GetPayloadSize() );
                mIteration++;
                mResultsView.InsertItem( mResultsView.GetItemCount(), now.Format( _T( "%Y-%m-%d %H.%M.%S" ) ) );
                }
            else
                {
                KillTimer( mTimer );
                mTimer = 0;
                setStarted( FALSE );
                }
            break;
            }
        }
}

//----------------------------------------------------------------------
//! \brief This function handles CDT Ping Receipts
//! \details As CDT Ping Receipts are received, this maps them to their
//!          original requests and records round-trip elapsed time and
//!          aggregates metrics on all receipts to update the display
//----------------------------------------------------------------------
LRESULT CCdtPingDlg::OnCDTPingReceived( WPARAM wParam, LPARAM lParam)
    {
    DWORD tickCount = GetTickCount();
    COleDateTime now = COleDateTime::GetCurrentTime();
    sint16 cdtPayloadSize = (sint16) wParam;
    uint8 const * cdtPayload = (uint8 const *) lParam;

    //must have at least the size of id field to be handled here
    if( cdtPayloadSize < sizeof( DWORD ) )
        {
        return 0;
        }
    DWORD id = 0;
    memcpy( &id, cdtPayload, sizeof( DWORD ) );
    for( int i=0; i<mResults.GetCount(); i++ )
        {
        if( mResults[i]->GetId() == id )
            {
            mResults[i]->CompareResponse( tickCount, cdtPayloadSize, cdtPayload );
            mResultsView.SetItemText( i, 1, now.Format( _T( "%Y-%m-%d %H.%M.%S" ) ) );
            CString elapsed;
            elapsed.Format( _T( "%d" ), mResults[i]->GetElapsedTimeMillis() );
            mResultsView.SetItemText( i, 2, elapsed );
            mResultsView.SetItemText( i, 3, mResults[i]->IsResponseMatched() ? _T( "yes" ) : _T( "no" ) );
            break;
            }
        }

    double totalElapsed = 0;
    int totalDelivered = 0;
    int totalMismatched = 0;
    for( int i=0; i<mResults.GetCount(); i++ )
        {
        if( mResults[i]->IsResponded() )
            {
            totalDelivered++;
            totalElapsed += mResults[i]->GetElapsedTimeMillis();
            if( !mResults[i]->IsResponseMatched() )
                {
                totalMismatched++;
                }
            }
        }
    CString delivered;
    delivered.Format( _T( "%d/%d" ), totalDelivered, mResults.GetCount() );
    GetDlgItem( IDC_TOTAL_DELIVERED_STATIC )->SetWindowText( delivered );
    if( totalDelivered > 0 )
        {
        CString avgElapsed;
        avgElapsed.Format( _T( "%.0f" ), totalElapsed / (double) totalDelivered );
        GetDlgItem( IDC_AVG_ELAPSED_TIME_STATIC )->SetWindowText( avgElapsed );
        }
    CString mismatched;
    mismatched.Format( _T( "%d" ), totalMismatched );
    GetDlgItem( IDC_TOTAL_MISMATCHED_STATIC )->SetWindowText( mismatched );
    return 0;
    }

static COleDateTime epoch(1970,1,1,0,0,0);

//----------------------------------------------------------------------
//! \brief Initialize payload with randomized data
//----------------------------------------------------------------------
CdtPingTestPacket::CdtPingTestPacket( UINT payloadSize ) :
        mPayloadSize( min( sizeof( mPayload ) - sizeof( DWORD ), payloadSize ) ),
        mRespondedOn( 0 ),
        mResponseMatched( FALSE )
    {
        mId = GetTickCount();
        memcpy( mPayload, &mId, sizeof( DWORD ) );
        for( int i=0; i<mPayloadSize; i++ )
            {
            mPayload[i + sizeof( DWORD )] = rand() % 256;
            }
        mPayloadSize += sizeof( DWORD );
    }

CdtPingTestPacket::~CdtPingTestPacket() {}

//----------------------------------------------------------------------
//! \brief Compare receipt data with original payload
//----------------------------------------------------------------------
void CdtPingTestPacket::CompareResponse( DWORD respondedOn, const uint8 rightSize, const uint8 * right )
    {
    mRespondedOn = respondedOn;
    if( rightSize == mPayloadSize )
        {
        for( int i=0; i<mPayloadSize; i++ )
            {
            if( mPayload[i] != right[i] )
                {
                return;
                }
            }
        mResponseMatched = TRUE;
        }
    }

//----------------------------------------------------------------------
//! \brief Calculate elapsed time from sent and responded time
//----------------------------------------------------------------------
int CdtPingTestPacket::GetElapsedTimeMillis()
    {
    if( mRespondedOn != 0 )
        {
        return mRespondedOn - mId;
        }

    return 0;
    }

#endif
