/*********************************************************************
*
*   HEADER NAME:
*       CCdtPingDlg.h
*
* Copyright 2014 by Garmin Ltd. or its subsidiaries.
*---------------------------------------------------------------------
* $NoKeywords$
*********************************************************************/
#pragma once

#include "FmiApplicationLayer.h"
#include "CWndEventListener.h"
#include <afxtempl.h>
#include "afxcmn.h"

#if( CDT_SUPPORT )

//----------------------------------------------------------------------
//! \brief Container for CDT Ping test information
//! \details This class holds a randomized payload of configurable size
//!          as well as delivery confirmation information for metrics
//!          gathering purposes
//----------------------------------------------------------------------
class CdtPingTestPacket
{
public:
    CdtPingTestPacket( UINT payloadSize );

    virtual ~CdtPingTestPacket();

    const DWORD GetId() const { return mId; }
    const uint8 * GetPayload() const { return mPayload; }
    const uint8 GetPayloadSize() const { return mPayloadSize; }

    void CompareResponse( DWORD respondedOn, const uint8 rightSize, const uint8 * right );
    const BOOL IsResponded() const { return mRespondedOn != 0; }
    const BOOL IsResponseMatched() const { return mResponseMatched; }
    int GetElapsedTimeMillis();

protected:
    DWORD mId;
    uint8 mPayloadSize;
    uint8 mPayload[ 254 ];
    DWORD mRespondedOn;
    BOOL mResponseMatched;

};

// CCdtPingDlg dialog

//----------------------------------------------------------------------
//! \brief Run a CDT Ping loop and evaluate results
//! \details
//----------------------------------------------------------------------
class CCdtPingDlg : public CDialog, public CWndEventListener
{
    DECLARE_DYNAMIC(CCdtPingDlg)

public:
    CCdtPingDlg(
        CWnd* pParent,
        FmiApplicationLayer & aCom
    );
    virtual ~CCdtPingDlg();

// Dialog Data
    enum { IDD = IDD_CDT_PING };

protected:
    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

    DECLARE_MESSAGE_MAP()

    BOOL OnInitDialog();

    afx_msg void OnTimer( UINT aTimerId );

    afx_msg LRESULT OnCDTPingReceived( WPARAM, LPARAM );

    void setStarted( BOOL started );

    //! Reference to the FMI communication controller
    FmiApplicationLayer& mCom;

    // loop state
    UINT mTimer;
    UINT mIteration;

    // results
    CArray<CdtPingTestPacket*,CdtPingTestPacket*> mResults;

    // loop control parameters
    UINT mInterval;
    UINT mPayloadSize;
    UINT mRepeatIterations;
    afx_msg void OnBnClickedToggleProcess();
public:
    BOOL mRepeatFixed;
    CListCtrl mResultsView;
    afx_msg void OnBnClickedClearResults();
    BOOL mClearOnStart;
    };

#endif