/*********************************************************************
*
*   MODULE NAME:
*       CDashcamDlg.cpp
*
* Copyright 2015 by Garmin Ltd. or its subsidiaries.
*---------------------------------------------------------------------
* $NoKeywords$
*********************************************************************/
#include "stdafx.h"
#include "CDashcamDlg.h"
#include "Event.h"
#include "util.h"

#if( FMI_SUPPORT_A622 )

// CDashcamDlg dialog

IMPLEMENT_DYNAMIC(CDashcamDlg, CDialog)

BEGIN_MESSAGE_MAP(CDashcamDlg, CDialog)
    ON_MESSAGE( WM_EVENT( EVENT_FMI_DASHCAM_LOCK_RECEIPT ), OnDashcamLockReceipt )
    ON_MESSAGE( WM_EVENT( EVENT_FMI_DASHCAM_CONFIG_RECEIPT ), OnDashcamConfigReceipt )
    ON_MESSAGE( WM_EVENT( EVENT_FMI_DASHCAM_GET_SETTINGS_RESPONSE ), OnDashcamGetSettingsResponse )
    ON_BN_CLICKED( IDC_LOCK_DASHCAM_SETTINGS, OnReevaluateLock )
    ON_BN_CLICKED( IDC_UNLOCK_DASHCAM_SETTINGS, OnReevaluateLock )
    ON_BN_CLICKED( IDC_LOCK_ASSIST_SETTINGS, OnReevaluateLock )
    ON_BN_CLICKED( IDC_UNLOCK_ASSIST_SETTINGS, OnReevaluateLock )
    ON_BN_CLICKED( IDC_ENABLE_FORWARD_COLLISION, OnReevaluate )
    ON_BN_CLICKED( IDC_DISABLE_FORWARD_COLLISION, OnReevaluate )
    ON_LBN_SELCHANGE( IDC_FWD_COLLISION_SENSITIVITY, OnReevaluate )
    ON_BN_CLICKED( IDC_ENABLE_LANE_DEPARTURE_WARNING, OnReevaluate )
    ON_BN_CLICKED( IDC_DISABLE_LANE_DEPARTURE_WARNING, OnReevaluate )
    ON_BN_CLICKED( IDC_ENABLE_RECORD_ON_STARTUP, OnReevaluate )
    ON_BN_CLICKED( IDC_DISABLE_RECORD_ON_STARTUP, OnReevaluate )
    ON_BN_CLICKED( IDC_ENABLE_INCIDENT_DETECTION, OnReevaluate )
    ON_BN_CLICKED( IDC_DISABLE_INCIDENT_DETECTION, OnReevaluate )
    ON_BN_CLICKED( IDC_ENABLE_RECORD_AUDIO, OnReevaluate )
    ON_BN_CLICKED( IDC_DISABLE_RECORD_AUDIO, OnReevaluate )
    ON_BN_CLICKED( IDC_ENABLE_DATA_OVERLAY_DATE_TIME, OnReevaluate )
    ON_BN_CLICKED( IDC_DISABLE_DATA_OVERLAY_DATE_TIME, OnReevaluate )
    ON_BN_CLICKED( IDC_ENABLE_DATA_OVERLAY_LOCATION_SPEED, OnReevaluate )
    ON_BN_CLICKED( IDC_DISABLE_DATA_OVERLAY_LOCATION_SPEED, OnReevaluate )
    ON_BN_CLICKED( IDC_MEDIUM_VIDEO_RESOLUTION, OnReevaluate )
    ON_BN_CLICKED( IDC_HIGH_VIDEO_RESOLUTION, OnReevaluate )
    ON_LBN_SELCHANGE( IDC_RECORD_AFTER_POWER_LOSS, OnReevaluate )
    ON_BN_CLICKED( IDC_DASHCAM_REFRESH_SETTINGS, OnRefreshSettings )
    ON_BN_CLICKED( IDOK, OnBnClickedSet )
    ON_BN_CLICKED( IDC_DASHCAM_SEND_SELECT_ALL, OnBnClickedSelectAll )
    ON_BN_CLICKED( IDCANCEL, OnClose )
    ON_BN_CLICKED( IDC_DASHCAM_SET_LOCK, OnBnClickedSetLock )
    ON_BN_CLICKED( IDC_DASHCAM_SEND_SELECT_ALL, OnReevaluate )
    ON_BN_CLICKED( IDC_DASHCAM_SEND_COLLISION_WARNING, OnReevaluate )
    ON_BN_CLICKED( IDC_DASHCAM_SEND_COLLISION_SENSITIVITY, OnReevaluate )
    ON_BN_CLICKED( IDC_DASHCAM_SEND_LANE_DEPARTURE, OnReevaluate )
    ON_BN_CLICKED( IDC_DASHCAM_SEND_RECORD_ON_STARTUP, OnReevaluate )
    ON_BN_CLICKED( IDC_DASHCAM_SEND_INCIDENT_DETECTION, OnReevaluate )
    ON_BN_CLICKED( IDC_DASHCAM_SEND_RECORD_AUDIO, OnReevaluate )
    ON_BN_CLICKED( IDC_DASHCAM_SEND_DATA_OVERLAY_DATE_TIME, OnReevaluate )
    ON_BN_CLICKED( IDC_DASHCAM_SEND_DATA_OVERLAY_LOCATION_SPEED, OnReevaluate )
    ON_BN_CLICKED( IDC_DASHCAM_SEND_VIDEO_RESOLUTION, OnReevaluate )
    ON_BN_CLICKED( IDC_DASHCAM_SEND_RECORD_AFTER_POWER_LOSS, OnReevaluate )

END_MESSAGE_MAP()

//----------------------------------------------------------------------
//! \brief Constructor
//! \param aParent The parent of this dialog
//! \param aCom Reference to the FMI communication controller
//----------------------------------------------------------------------
CDashcamDlg::CDashcamDlg
    (
    CWnd                * aParent,
    FmiApplicationLayer & aCom
    )
    : CDialog( CDashcamDlg::IDD, aParent )
    , mCom( aCom )
    , mLockDashcam( -1 )
    , mLockAssist( -1 )
    , mForwardCollision( -1 )
    , mCollisionSensitivity( -1 )
    , mLaneDeparture( -1 )
    , mRecordOnStartup( -1 )
    , mIncidentDetection( -1 )
    , mRecordAudio( -1 )
    , mDataOverlayDateTime( -1 )
    , mDataOverlayLocationSpeed( -1 )
    , mVideoResolution( -1 )
    , mRecordAfterPowerLoss( -1 )
    , mSelectAll( FALSE )
    , mSendCollisionWarning( FALSE )
    , mSendCollisionSensitivity( FALSE )
    , mSendLaneDeparture( FALSE )
    , mSendRecordOnStartup( FALSE )
    , mSendIncidentDetection( FALSE )
    , mSendRecordAudio( FALSE )
    , mSendDataOverlayDateTime( FALSE )
    , mSendDataOverlayLocationSpeed( FALSE )
    , mSendVideoResolution( FALSE )
    , mSendRecordAfterPowerLoss( FALSE )
{
}

//----------------------------------------------------------------------
//! \brief Destructor
//----------------------------------------------------------------------
CDashcamDlg::~CDashcamDlg()
{
}

//----------------------------------------------------------------------
//! \brief Initialize the dialog
//! \details This function is called when the window is created. It
//!     sets up the parent, so it can get info from and send a message
//!     to FmiApplicationLayer.
//! \return TRUE, since this function does not set focus to a control
//----------------------------------------------------------------------
BOOL CDashcamDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    UpdateData( FALSE );
    return TRUE;
}   /* OnInitDialog() */

//----------------------------------------------------------------------
//! \brief Perform dialog data exchange and validation
//! \param  aDataExchange The DDX context
//----------------------------------------------------------------------
void CDashcamDlg::DoDataExchange
    (
    CDataExchange * aDataExchange
    )
{
    CDialog::DoDataExchange( aDataExchange );

    DDX_Radio( aDataExchange, IDC_UNLOCK_DASHCAM_SETTINGS, mLockDashcam );
    DDX_Radio( aDataExchange, IDC_UNLOCK_ASSIST_SETTINGS, mLockAssist );
    DDX_Radio( aDataExchange, IDC_DISABLE_FORWARD_COLLISION, mForwardCollision );
    DDX_CBIndex( aDataExchange, IDC_FWD_COLLISION_SENSITIVITY, mCollisionSensitivity );
    DDX_Radio( aDataExchange, IDC_DISABLE_LANE_DEPARTURE_WARNING, mLaneDeparture );
    DDX_Radio( aDataExchange, IDC_DISABLE_RECORD_ON_STARTUP, mRecordOnStartup );
    DDX_Radio( aDataExchange, IDC_DISABLE_INCIDENT_DETECTION, mIncidentDetection );
    DDX_Radio( aDataExchange, IDC_DISABLE_RECORD_AUDIO, mRecordAudio );
    DDX_Radio( aDataExchange, IDC_DISABLE_DATA_OVERLAY_DATE_TIME, mDataOverlayDateTime );
    DDX_Radio( aDataExchange, IDC_DISABLE_DATA_OVERLAY_LOCATION_SPEED, mDataOverlayLocationSpeed );
    DDX_Radio( aDataExchange, IDC_HIGH_VIDEO_RESOLUTION, mVideoResolution );
    DDX_CBIndex( aDataExchange, IDC_RECORD_AFTER_POWER_LOSS, mRecordAfterPowerLoss );
    DDX_Check( aDataExchange, IDC_DASHCAM_SEND_SELECT_ALL, mSelectAll );
    DDX_Check( aDataExchange, IDC_DASHCAM_SEND_COLLISION_WARNING, mSendCollisionWarning );
    DDX_Check( aDataExchange, IDC_DASHCAM_SEND_COLLISION_SENSITIVITY, mSendCollisionSensitivity );
    DDX_Check( aDataExchange, IDC_DASHCAM_SEND_LANE_DEPARTURE, mSendLaneDeparture );
    DDX_Check( aDataExchange, IDC_DASHCAM_SEND_RECORD_ON_STARTUP, mSendRecordOnStartup );
    DDX_Check( aDataExchange, IDC_DASHCAM_SEND_INCIDENT_DETECTION, mSendIncidentDetection );
    DDX_Check( aDataExchange, IDC_DASHCAM_SEND_RECORD_AUDIO, mSendRecordAudio );
    DDX_Check( aDataExchange, IDC_DASHCAM_SEND_DATA_OVERLAY_DATE_TIME, mSendDataOverlayDateTime );
    DDX_Check( aDataExchange, IDC_DASHCAM_SEND_DATA_OVERLAY_LOCATION_SPEED, mSendDataOverlayLocationSpeed );
    DDX_Check( aDataExchange, IDC_DASHCAM_SEND_VIDEO_RESOLUTION, mSendVideoResolution );
    DDX_Check( aDataExchange, IDC_DASHCAM_SEND_RECORD_AFTER_POWER_LOSS, mSendRecordAfterPowerLoss );
}

//----------------------------------------------------------------------
//! \brief Button handler for the Set Lock button pressed
//! \details Calls FmiApplicationLayer to initiate the lock settings
//!     request protocol
//----------------------------------------------------------------------
void CDashcamDlg::OnBnClickedSetLock()
{
    fmi_dashcam_lock_settings_request_data_type message;
    memset( &message, 0, sizeof( message ) );
    UpdateData();

    if( -1 != mLockDashcam && -1 != mLockAssist )
    {
        message.origination_time = UTIL_get_current_garmin_time();
        message.lock_change = (boolean) mLockDashcam;
        message.lock_change_adas = (boolean) mLockAssist;
        mCom.sendFmiPacket( FMI_DASHCAM_SETTINGS_LOCK_REQUEST, (uint8*)&message, sizeof( message ) );
    }
}

//----------------------------------------------------------------------
//! \brief Button handler for the Refresh Settings button
//! \details Calls FmiApplicationLayer to initiate the Get settings request
//!     protocol
//----------------------------------------------------------------------
void CDashcamDlg::OnRefreshSettings()
{
    UpdateData( TRUE );
    fmi_dashcam_get_settings_request_data_type getSettings;
    getSettings.change_id = GetTickCount();
    mCom.sendFmiPacket( FMI_DASHCAM_GET_SETTINGS_REQUEST, (uint8*)&getSettings, sizeof( getSettings) );
    UpdateData( FALSE );
}

//----------------------------------------------------------------------
//! \brief Button handler for the Set button
//! \details Retrieves configuration values from user interface and
//!         sends dashcam configuraiton request protocol
//----------------------------------------------------------------------
void CDashcamDlg::OnBnClickedSet()
{
    fmi_dashcam_set_config_request_data_type setConfig;
    memset( &setConfig, 0, sizeof( setConfig ) );
    setConfig.settings_count = 0;

    if( -1 != mForwardCollision && mSendCollisionWarning )
    {
        setConfig.change_id = GetTickCount();
        setConfig.settings_list[setConfig.settings_count].setting_id = DASHCAM_FORWARD_COLLISION;
        setConfig.settings_list[setConfig.settings_count].new_value = (uint8) mForwardCollision;
        setConfig.settings_count++;
    }

    if( -1 != mCollisionSensitivity && mSendCollisionSensitivity )
    {
        setConfig.change_id = GetTickCount();
        setConfig.settings_list[setConfig.settings_count].setting_id = DASHCAM_COLLISION_SENSITIVITY;
        setConfig.settings_list[setConfig.settings_count].new_value = (uint8) mCollisionSensitivity;
        setConfig.settings_count++;
    }

    if( -1 != mLaneDeparture && mSendLaneDeparture )
    {
        setConfig.change_id = GetTickCount();
        setConfig.settings_list[setConfig.settings_count].setting_id = DASHCAM_LANE_DEPARTURE;
        setConfig.settings_list[setConfig.settings_count].new_value = (uint8) mLaneDeparture;
        setConfig.settings_count++;
    }

    if( -1 != mRecordOnStartup && mSendRecordOnStartup )
    {
        setConfig.change_id = GetTickCount();
        setConfig.settings_list[setConfig.settings_count].setting_id = DASHCAM_RECORD_ON_STARTUP;
        setConfig.settings_list[setConfig.settings_count].new_value = (uint8) mRecordOnStartup;
        setConfig.settings_count++;
    }

    if( -1 != mIncidentDetection && mSendIncidentDetection )
    {
        setConfig.change_id = GetTickCount();
        setConfig.settings_list[setConfig.settings_count].setting_id = DASHCAM_INCIDENT_DETECTION;
        setConfig.settings_list[setConfig.settings_count].new_value = (uint8) mIncidentDetection;
        setConfig.settings_count++;
    }

    if( -1 != mRecordAudio && mSendRecordAudio )
    {
        setConfig.change_id = GetTickCount();
        setConfig.settings_list[setConfig.settings_count].setting_id = DASHCAM_RECORD_AUDIO;
        setConfig.settings_list[setConfig.settings_count].new_value = (uint8) mRecordAudio;
        setConfig.settings_count++;
    }

    if( -1 != mDataOverlayDateTime && mSendDataOverlayDateTime )
    {
        setConfig.change_id = GetTickCount();
        setConfig.settings_list[setConfig.settings_count].setting_id = DASHCAM_DATA_OVERLAY_DATE_TIME;
        setConfig.settings_list[setConfig.settings_count].new_value = (uint8) mDataOverlayDateTime;
        setConfig.settings_count++;
    }

    if( -1 != mDataOverlayLocationSpeed && mSendDataOverlayLocationSpeed )
    {
        setConfig.change_id = GetTickCount();
        setConfig.settings_list[setConfig.settings_count].setting_id = DASHCAM_DATA_OVERLAY_LOCATION_SPEED;
        setConfig.settings_list[setConfig.settings_count].new_value = (uint8) mDataOverlayLocationSpeed;
        setConfig.settings_count++;
    }

    if( -1 != mVideoResolution && mSendVideoResolution)
    {
        setConfig.change_id = GetTickCount();
        setConfig.settings_list[setConfig.settings_count].setting_id = DASHCAM_VIDEO_RESOLUTION;
        setConfig.settings_list[setConfig.settings_count].new_value = (uint8) mVideoResolution;
        setConfig.settings_count++;
    }

    if( -1 != mRecordAfterPowerLoss && mSendRecordAfterPowerLoss )
    {
        setConfig.change_id = GetTickCount();
        setConfig.settings_list[setConfig.settings_count].setting_id = DASHCAM_RECORD_AFTER_POWER_LOSS;
        setConfig.settings_list[setConfig.settings_count].new_value = (uint8) mRecordAfterPowerLoss;
        setConfig.settings_count++;
    }

    if( setConfig.settings_count )
    {
        mCom.sendFmiPacket( FMI_DASHCAM_CONFIG_REQUEST, (uint8*)&setConfig, sizeof( setConfig ) );
    }
}    /* OnBnClickedSet () */

//----------------------------------------------------------------------
//! \brief Handler for the select all check
//! \details selects or deselects all send boxes
//----------------------------------------------------------------------
void CDashcamDlg::OnBnClickedSelectAll()
{
    UpdateData( TRUE );
    mSendCollisionWarning = mSelectAll;
    mSendCollisionSensitivity = mSelectAll;
    mSendLaneDeparture = mSelectAll;
    mSendRecordOnStartup = mSelectAll;
    mSendIncidentDetection = mSelectAll;
    mSendRecordAudio = mSelectAll;
    mSendDataOverlayDateTime = mSelectAll;
    mSendDataOverlayLocationSpeed = mSelectAll;
    mSendVideoResolution = mSelectAll;
    mSendRecordAfterPowerLoss = mSelectAll;
    UpdateData( FALSE );

    OnReevaluate();
}

//----------------------------------------------------------------------
//! \brief Handler for the Close action
//! \details Closes the window.
//----------------------------------------------------------------------
void CDashcamDlg::OnClose()
{
    UpdateData( TRUE );
    mLockDashcam = -1;
    mLockAssist = -1;
    mForwardCollision = -1;
    mCollisionSensitivity = -1;
    mLaneDeparture = -1;
    mRecordOnStartup = -1;
    mIncidentDetection = -1;
    mRecordAudio = -1;
    mDataOverlayDateTime = -1;
    mDataOverlayLocationSpeed = -1;
    mVideoResolution = -1;
    mRecordAfterPowerLoss = -1;
    mSelectAll = FALSE;
    mSendCollisionWarning = FALSE;
    mSendCollisionSensitivity = FALSE;
    mSendLaneDeparture = FALSE;
    mSendRecordOnStartup = FALSE;
    mSendIncidentDetection = FALSE;
    mSendRecordAudio = FALSE;
    mSendDataOverlayDateTime = FALSE;
    mSendDataOverlayLocationSpeed = FALSE;
    mSendVideoResolution = FALSE;
    mSendRecordAfterPowerLoss = FALSE;
    UpdateData( FALSE );

    CDialog::OnCancel();
}    /* OnClose() */

//----------------------------------------------------------------------
//! \brief Called by MFC after the window has been destroyed; performs
//!     final termination activities.
//----------------------------------------------------------------------
void CDashcamDlg::PostNcDestroy()
{
    CDialog::PostNcDestroy();
}   /* PostNcDestroy() */

//----------------------------------------------------------------------
//! \brief This function handles the dashcam lock/unlock receipt event from device
//----------------------------------------------------------------------
LRESULT CDashcamDlg::OnDashcamLockReceipt ( WPARAM return_code, LPARAM )
{
    CString message;

    if( DASHCAM_SUCCESS == return_code )
    {
        message.Format( _T( "Camera settings successfully locked/unlocked" ) );
        MessageBox( message, _T( "Camera Settings Success" ), MB_OK | MB_ICONINFORMATION );
    }
    else
    {
        postFailure( (uint8) return_code );
    }
    return 0;
}

//----------------------------------------------------------------------
//! \brief This function handles the dashcam set configuration receipt event from device
//----------------------------------------------------------------------
LRESULT CDashcamDlg::OnDashcamConfigReceipt ( WPARAM return_code, LPARAM )
{
    CString message;

    if( DASHCAM_SUCCESS == return_code )
    {
        message.Format( _T( "Camera settings configuration successfully updated." ) );
        MessageBox( message, _T( "Camera Settings Success" ), MB_OK | MB_ICONINFORMATION );
    }
    else
    {
        postFailure( (uint8) return_code );
    }
    return 0;
}

//----------------------------------------------------------------------
//! \brief This function handles dashcam get settings receipt event from device
//----------------------------------------------------------------------
LRESULT CDashcamDlg::OnDashcamGetSettingsResponse ( WPARAM return_code, LPARAM response_data )
{
    CString message;

    if( DASHCAM_SUCCESS == return_code )
    {
        message.Format( _T( "Camera settings successfully updated." ) );
        MessageBox( message, _T( "Refresh Settings Success" ), MB_OK | MB_ICONINFORMATION );

        fmi_dashcam_get_settings_response_data_type messageEvent = *( (fmi_dashcam_get_settings_response_data_type* ) response_data );

        for( int i=0; i<messageEvent.settings_count; i++ )
        {
            UpdateData( TRUE );

            switch(messageEvent.settings_list[i].setting_id)
            {
            case DASHCAM_FORWARD_COLLISION:
                {
                mForwardCollision = messageEvent.settings_list[i].new_value;
                break;
                }
            case DASHCAM_COLLISION_SENSITIVITY:
                {
                mCollisionSensitivity = messageEvent.settings_list[i].new_value;
                break;
                }
            case DASHCAM_LANE_DEPARTURE:
                {
                mLaneDeparture = messageEvent.settings_list[i].new_value;
                break;
                }
            case DASHCAM_RECORD_ON_STARTUP:
                {
                mRecordOnStartup = messageEvent.settings_list[i].new_value;
                break;
                }
            case DASHCAM_INCIDENT_DETECTION:
                {
                mIncidentDetection = messageEvent.settings_list[i].new_value;
                break;
                }
            case DASHCAM_RECORD_AUDIO:
                {
                mRecordAudio = messageEvent.settings_list[i].new_value;
                break;
                }
            case DASHCAM_DATA_OVERLAY_DATE_TIME:
                {
                mDataOverlayDateTime = messageEvent.settings_list[i].new_value;
                break;
                }
            case DASHCAM_DATA_OVERLAY_LOCATION_SPEED:
                {
                mDataOverlayLocationSpeed = messageEvent.settings_list[i].new_value;
                break;
                }
            case DASHCAM_VIDEO_RESOLUTION:
                {
                mVideoResolution = messageEvent.settings_list[i].new_value;
                break;
                }
            case DASHCAM_RECORD_AFTER_POWER_LOSS:
                {
                mRecordAfterPowerLoss = messageEvent.settings_list[i].new_value;
                break;
                }
            }
            UpdateData( FALSE );
        }
    }
    else
    {
        postFailure( (uint8) return_code );
    }
    return 0;
}

//----------------------------------------------------------------------
//! \brief Change handler for editable controls
//! \details Make the SET button clickable if the data is valid for sending
//----------------------------------------------------------------------
void CDashcamDlg::OnReevaluate()
{
    UpdateData();
    //checks to make sure a send is checked with a corrosponding setting entered.
    //if so, set button enabled, if not, set button disabled.
    if( ( mSendCollisionWarning && ( -1 != mForwardCollision ) ) ||
        ( mSendCollisionSensitivity && ( -1 != mCollisionSensitivity ) ) ||
        ( mSendLaneDeparture && ( -1 != mLaneDeparture ) ) ||
        ( mSendRecordOnStartup && ( -1 != mRecordOnStartup ) ) ||
        ( mSendIncidentDetection && ( -1 != mIncidentDetection ) ) ||
        ( mSendRecordAudio && ( -1 != mRecordAudio ) ) ||
        ( mSendDataOverlayDateTime && ( -1 != mDataOverlayDateTime ) ) ||
        ( mSendDataOverlayLocationSpeed && ( -1 != mDataOverlayLocationSpeed ) ) ||
        ( mSendVideoResolution && ( -1 != mVideoResolution ) ) ||
        ( mSendRecordAfterPowerLoss && ( -1 != mRecordAfterPowerLoss ) ) )
    {
        GetDlgItem( IDOK )->EnableWindow();
    }
    else
    {
        GetDlgItem( IDOK )->EnableWindow( FALSE );
    }
}

//----------------------------------------------------------------------
//! \brief Change handler for editable controls
//! \details Make the Set Lock/Unlock button clickable if both locks/unlocks selected
//----------------------------------------------------------------------
void CDashcamDlg::OnReevaluateLock()
{
    UpdateData();

    if( ( -1 != mLockDashcam ) &&
        ( -1 != mLockAssist ) )
    {
        GetDlgItem( IDC_DASHCAM_SET_LOCK )->EnableWindow();
    }
    else
    {
        GetDlgItem( IDC_DASHCAM_SET_LOCK )->EnableWindow( FALSE );
    }
}

void CDashcamDlg::postFailure( uint8 return_code )
{

    CString message;
    switch( return_code )
    {
        case DASHCAM_ERR_AFTER_POWER_LOSS_SETTINGS_OUT_OF_RANGE:
            {
            message.Format( _T( "Camera settings failure due to Power Loss Settings out of range." ) );
            MessageBox( message, _T( "Camera Settings Failure" ), MB_OK | MB_ICONINFORMATION );
            break;
            }
        case DASHCAM_ERR_RECORDER_RESOLUTION_OUT_OF_RANGE:
            {
            message.Format( _T( "Camera settings failure due to recorder resolution out of range." ) );
            MessageBox( message, _T( "Camera Settings Failure" ), MB_OK | MB_ICONINFORMATION );
            break;
            }
        case DASHCAM_ERR_FCWS_SENSITIVITY_OUT_OF_RANGE:
            {
            message.Format( _T( "Camera settings failure due to forward collision warning sensitivity out of range." ) );
            MessageBox( message, _T( "Camera Settings Failure" ), MB_OK | MB_ICONINFORMATION );
            break;
            }
        case DASHCAM_ERR_SETTING_ID_OUT_OF_RANGE:
            {
            message.Format( _T( "Camera settings failure due to setting ID out of range." ) );
            MessageBox( message, _T( "Camera Settings Failure" ), MB_OK | MB_ICONINFORMATION );
            break;
            }
        default:
            {
            message.Format( _T( "Camera settings failure." ) );
            MessageBox( message, _T( "Camera Settings Failure" ), MB_OK | MB_ICONINFORMATION );
            break;
            }
    }
}
#endif