/*********************************************************************
*
*   MODULE NAME:
*       CDashcamNtfctnFromClient.cpp
*
*   Copyright 2015 by Garmin Ltd. or its subsidiaries.
*---------------------------------------------------------------------
* $NoKeywords$
*********************************************************************/
#include "stdafx.h"
#include "FmiLogParser.h"
#include "CFmiApplication.h"
#include "CDashcamNtfctnFromClient.h"
#include "util.h"

#if( FMI_SUPPORT_A622 )

IMPLEMENT_DYNAMIC( CDashcamNtfctnFromClient, CDialog )

BEGIN_MESSAGE_MAP( CDashcamNtfctnFromClient, CDialog )
END_MESSAGE_MAP()

//----------------------------------------------------------------------
//! \brief Constructor
//! \param aParent The parent of this dialog
//! \param aCom Reference to the FMI communication controller
//! \param aEvent The notification received from the client
//----------------------------------------------------------------------
CDashcamNtfctnFromClient::CDashcamNtfctnFromClient
    (
    CWnd                                  * aParent,
    FmiApplicationLayer                   & aCom,
    const fmi_dashcam_ntfctn_req_data_type & aEvent
    )
    : CDialog( IDD_DASHCAM_NTFCTN_FROM_CLIENT, aParent )
    , mCom( aCom )
    , mOriginationTime( _T("") )
    , mType( _T("") )
    , mLongitude( _T("") )
    , mLatitude( _T("") )
    , mIncidentFilename( _T("") )

{
    init( aEvent.origination_time, aEvent.scposn, aEvent.ntfctn_type );
}

//----------------------------------------------------------------------
//! \brief Constructor
//! \param aParent The parent of this dialog
//! \param aCom Reference to the FMI communication controller
//! \param aEvent The notification received from the client
//----------------------------------------------------------------------
CDashcamNtfctnFromClient::CDashcamNtfctnFromClient
    (
    CWnd                                  * aParent,
    FmiApplicationLayer                   & aCom,
    const fmi_dashcam_ntfctn_incident_req_data_type & aEvent
    )
    : CDialog( IDD_DASHCAM_NTFCTN_FROM_CLIENT, aParent )
    , mCom( aCom )
    , mOriginationTime( _T("") )
    , mType( _T("") )
    , mLongitude( _T("") )
    , mLatitude( _T("") )
    , mIncidentFilename( _T("") )

{
    init( aEvent.origination_time, aEvent.scposn, aEvent.ntfctn_type );

    mIncidentFilename = CString( aEvent.incident_fname );
}

//----------------------------------------------------------------------
//! \brief Destructor
//----------------------------------------------------------------------
CDashcamNtfctnFromClient::~CDashcamNtfctnFromClient()
{
}

//----------------------------------------------------------------------
//! \brief Perform dialog data exchange and validation
//! \param  aDataExchange The DDX context
//----------------------------------------------------------------------
void CDashcamNtfctnFromClient::DoDataExchange
    (
    CDataExchange * aDataExchange
    )
{
    CDialog::DoDataExchange( aDataExchange );
    DDX_Text( aDataExchange, IDC_DASHCAM_NTFCTN_TIMESTAMP, mOriginationTime );
    DDX_Text( aDataExchange, IDC_DASHCAM_NTFCTN_TYPE, mType );
    DDX_Text( aDataExchange, IDC_DASHCAM_NTFCTN_LON, mLongitude );
    DDX_Text( aDataExchange, IDC_DASHCAM_NTFCTN_LAT, mLatitude );
    DDX_Text( aDataExchange, IDC_DASHCAM_NTFCTN_INCIDENT_FNAME, mIncidentFilename );
}

//----------------------------------------------------------------------
//! \brief Initialize the field data
//----------------------------------------------------------------------
void CDashcamNtfctnFromClient::init
    (
    time_type           origination_time,
    sc_position_type    scposn,
    uint8               ntfctn_type
    )
{
    TCHAR               stringBuffer[200];
    time_type           localTime;
    date_time_data_type localDateTime;
    char                localTimeString[13];

    UTIL_convert_UTC_to_local( &origination_time, &localTime );
    UTIL_convert_seconds_to_time_type( &localTime, &localDateTime );
    UTIL_format_time_string( &localDateTime, localTimeString, sizeof( localTimeString ) );

    MultiByteToWideChar( mCom.mClientCodepage, 0, localTimeString, -1, stringBuffer, 13 );
    stringBuffer[13] = '\0';
    mOriginationTime.Format( _T(" %s"), stringBuffer );

    switch( ntfctn_type )
    {
    case DASHCAM_INCIDENT_NTFCTN:
        {
        mType = _T(" Incident");
        break;
        }
    case DASHCAM_FCW_NTFCTN:
        {
        mType = _T(" Fwd Collision Warning");
        break;
        }
    case DASHCAM_LDW_NTFCTN:
        {
        mType = _T(" Lane Departure Warning");
        break;
        }
    default:
        {
        mType = _T(" Unrecognized" );
        break;
        }
    }

    if( scposn.lat != INVALID_LAT )
    {
        double longitudeDegrees = UTIL_convert_semicircles_to_degrees( scposn.lon );
        double latitudeDegrees = UTIL_convert_semicircles_to_degrees( scposn.lat );

        if( latitudeDegrees < 0 )
        {
            mLatitude.Format(  _T( "%08.6f S" ), latitudeDegrees * -1 );
        }
        else
        {
            mLatitude.Format(  _T( "%08.6f N" ), latitudeDegrees );
        }

        if( longitudeDegrees < 0 )
        {
            mLongitude.Format(  _T( "%08.6f W" ), longitudeDegrees * -1 );
        }
        else
        {
            mLongitude.Format(  _T( "%08.6f E" ), longitudeDegrees );
        }
    }
}
#endif
