/*********************************************************************
*
*   MODULE NAME:
*       CEtaModeDlg.cpp
*
*   Copyright 2008-2016 by Garmin Ltd. or its subsidiaries.
*---------------------------------------------------------------------
* $NoKeywords$
*********************************************************************/
#include "stdafx.h"

#include "CEtaModeDlg.h"

using namespace std;

IMPLEMENT_DYNAMIC( CEtaModeDlg, CDialog )

BEGIN_MESSAGE_MAP( CEtaModeDlg, CDialog )
    ON_BN_CLICKED( IDOK, OnBnClickedSend )
END_MESSAGE_MAP()

/*--------------------------------------------------------------------
LITERAL CONSTANTS
--------------------------------------------------------------------*/

//! Mode types that the client accepts
//! \see fmi.h for the enumeration
static CEtaModeDlg::mode_type mode_types[] =
{
    _T("0 - None"),
    _T("1 - Dispatch Stops"),                   // Default Mode
    _T("2 - Driver Initiated Stops"),
    _T("3 - Dispatch or Driver Initiated Stops")
};

//----------------------------------------------------------------------
//! \brief Constructor
//! \param aParent The parent of this dialog
//! \param aCom Reference to the FMI communication controller
//----------------------------------------------------------------------
CEtaModeDlg::CEtaModeDlg
    (
    CWnd                * aParent,
    FmiApplicationLayer & aCom
    )
    : CDialog( IDD_ETA_MODE, aParent )
    , mCom( aCom )
    , mMode( ETA_SETTING_DISPATCH )
{
}

//----------------------------------------------------------------------
//! \brief Destructor
//----------------------------------------------------------------------
CEtaModeDlg::~CEtaModeDlg()
{
}

//----------------------------------------------------------------------
//! \brief Perform dialog data exchange and validation
//! \param aDataExchange The DDX context
//----------------------------------------------------------------------
void CEtaModeDlg::DoDataExchange
    (
    CDataExchange * aDataExchange
    )
{
    CDialog::DoDataExchange( aDataExchange );

    int mode;
    DDX_CBIndex( aDataExchange, IDC_ETA_MODE_CBO, mode );
    mMode = ( mode & 0xff );
}

//----------------------------------------------------------------------
//! \brief Initialize the dialog
//! \details This function is called when the window is created. It
//!     sets up the parent, so it can get info from and send a message
//!     to FmiApplicationLayer.
//! \return TRUE, since this function does not set focus to a control
//----------------------------------------------------------------------
BOOL CEtaModeDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Populate mode type combobox
    CComboBox * cbo = (CComboBox *)GetDlgItem( IDC_ETA_MODE_CBO );
    cbo->ResetContent();
    for( int i = 0; i < cnt_of_array( mode_types ); i++ )
    {
        cbo->AddString( mode_types[i] );
    }
    cbo->SetCurSel( ETA_SETTING_DISPATCH );

    return TRUE;
} /* OnInitDialog() */

//----------------------------------------------------------------------
//! \brief Click handler for Send button.
//! \details Sends the ETA Mode Request packet with the mode selected
//!     by the user.
//----------------------------------------------------------------------
void CEtaModeDlg::OnBnClickedSend()
{
    UpdateData( TRUE );

    mCom.sendEtaModeRequest( mMode );

    OnOK();
}