/*********************************************************************
*
*   HEADER NAME:
*       CFileConverterDlg.h
*
*   Copyright 2008-2015 by Garmin Ltd. or its subsidiaries.
*---------------------------------------------------------------------
* $NoKeywords$
*********************************************************************/
#pragma once

#include "FmiApplicationLayer.h"

#if( FMI_SUPPORT_A622 )
//----------------------------------------------------------------------
//! \brief File conversion tool to support various GPS file formats
//! \details
//----------------------------------------------------------------------
class CFileConverterDlg : public CDialog
{
    DECLARE_DYNAMIC( CFileConverterDlg )

public:
    CFileConverterDlg
        (
        CWnd                * aParent,
        FmiApplicationLayer & aCom
        );
    virtual ~CFileConverterDlg();

    //! Defines conversion type
    typedef struct
        {
        uint8 index;
        CString text;
        CString extension;
        CString newExtension;
        CString filter;
        } conversion_type;

    // Dialog Data
    enum { IDD = IDD_FILECONVERTER };

protected:

    DECLARE_MESSAGE_MAP()

    virtual void DoDataExchange
        (
        CDataExchange* aDataExchange
        );

    BOOL OnInitDialog();
    afx_msg void OnEnChangeFile();
    afx_msg void OnBnClickedFind();
    afx_msg void OnBnClickedChoose();
    afx_msg void OnBnClickedOk();
    afx_msg void OnComboChanged();

    CString FormatArguments( CString dstFilePath );
    void ReadFromPipe(void);
    BOOL CreateChildProcess(CString dstFilePath);

    //! Reference to the FMI communication controller
    FmiApplicationLayer& mCom;

    //! Path and file name of the file to be converted, as specified
    //!     by the user
    CString mSrcFilePath;

    //! Type of file conversion to be attempted, as specified by the user
    int mConversionType;

    //! Index of the chosen combo box item
    int mConversionIndex;

    //! File name of the converted file specified by the user.
    //!     Defaults as the same name as Src file with new extension.
    CString mDstFileName;

    //! Destination folder for the converted file.
    //!     Defaults to the same folder as Src file.
    CString mDstFolder;

    //! Unique Stop ID for the PSR
    CString mStopId;

    //! Route name (Stop Text) of the PSR (limited to 200 bytes)
    CString mRouteName;

    //! Flags identify the format of the the source and destination files coords
    BOOL mSrcCoords;
    BOOL mDstCoords;


    //! Handles for the redirection of the child process stdout
    HANDLE hChildStd_OUT_Rd;
    HANDLE hChildStd_OUT_Wr;

};
#endif