/*********************************************************************
*
*   MODULE NAME:
*       CFileTransferProgressDlg.cpp
*
*   Copyright 2008-2011 by Garmin Ltd. or its subsidiaries.
*---------------------------------------------------------------------
* $NoKeywords$
*********************************************************************/
#include "stdafx.h"
#include "CFmiApplication.h"
#include "CFileTransferProgressDlg.h"
#include "CFmiPcAppDlg.h"
#include "Event.h"

IMPLEMENT_DYNAMIC( CFileTransferProgressDlg, CDialog )

BEGIN_MESSAGE_MAP( CFileTransferProgressDlg, CDialog )
    ON_BN_CLICKED( IDC_FILE_PROGRESS_BTN_STOP, OnBnClickedStop )
    ON_MESSAGE( WM_EVENT( EVENT_FMI_FILE_TRANSFER_PROGRESS ), OnFileTransferProgress )
    ON_MESSAGE( WM_EVENT( EVENT_FMI_FILE_TRANSFER_STATE_CHANGED ), OnFileTransferStateChange )
END_MESSAGE_MAP()

//----------------------------------------------------------------------
//! \brief Constructor
//! \param aParent The parent of this dialog
//! \param aCom Reference to the FMI communication controller
//----------------------------------------------------------------------
CFileTransferProgressDlg::CFileTransferProgressDlg
    (
    CWnd                * aParent,
    FmiApplicationLayer & aCom
    )
    : CDialog( IDD_FILE_TRANSFER_PROGRESS, aParent )
    , mCom( aCom )
    , mTransferStatus( _T("") )
{
    mStopButtonClicked = FALSE;
}

//----------------------------------------------------------------------
//! \brief Destructor
//----------------------------------------------------------------------
CFileTransferProgressDlg::~CFileTransferProgressDlg()
{
}

//----------------------------------------------------------------------
//! \brief Perform dialog data exchange and validation
//! \param  aDataExchange The DDX context
//----------------------------------------------------------------------
void CFileTransferProgressDlg::DoDataExchange
    (
    CDataExchange * aDataExchange
    )
{
    CDialog::DoDataExchange( aDataExchange );
    DDX_Text( aDataExchange, IDC_FILE_PROGRESS_TXT_STATUS, mTransferStatus );
    DDX_Control( aDataExchange, IDC_FILE_PROGRESS_PROGRESS_BAR, mProgressBar );
}

//----------------------------------------------------------------------
//! \brief Initialize the dialog
//! \details This function is called when the window is created. It
//!     initializes the progress bar and text box.
//! \return TRUE, since this function does not set focus to a control
//----------------------------------------------------------------------
BOOL CFileTransferProgressDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    // display the current mTransferStatus
    OnFileTransferStateChange( 0, NULL );

    return TRUE;
} /* OnInitDialog() */

//----------------------------------------------------------------------
//! \brief Handle a File Transfer State Change event
//! \details Updates the transfer mTransferStatus text box (based on
//!     the current state known to FmiApplicationLayer) and, if the
//!     transfer is in progress, the progress bar.
//----------------------------------------------------------------------
afx_msg LRESULT CFileTransferProgressDlg::OnFileTransferStateChange( WPARAM, LPARAM )
{
    switch ( mCom.mFileTransferState )
    {
    //set mTransferStatus
    case FmiApplicationLayer::TRANSFER_NOT_STARTED:
        GetDlgItem( IDOK )->EnableWindow( TRUE );
        mTransferStatus.Format( _T("No Transfer") );
        break;
    case FmiApplicationLayer::TRANSFER_STARTED:
        GetDlgItem( IDOK )->EnableWindow( FALSE );
        mTransferStatus.Format( _T("Starting Transfer") );
        break;
    case FmiApplicationLayer::TRANSFER_IN_PROGRESS:
        GetDlgItem( IDOK )->EnableWindow( FALSE );
        mProgressBar.SetRange32( 0, mCom.mFileTransferSize );
        mProgressBar.SetPos( 0 );
        break;
    case FmiApplicationLayer::TRANSFER_COMPLETED:
        GetDlgItem( IDOK )->EnableWindow( TRUE );
        GetDlgItem( IDC_FILE_PROGRESS_BTN_STOP )->EnableWindow( FALSE );
        mTransferStatus.Format( _T("Done") );
        break;
    case FmiApplicationLayer::TRANSFER_FAILED:
        GetDlgItem( IDC_FILE_PROGRESS_BTN_STOP )->EnableWindow( FALSE );
        if( mStopButtonClicked )
            mTransferStatus.Format( _T("Canceled at user request") );
        else
            mTransferStatus.Format( _T("Failed") );
        GetDlgItem( IDOK )->ShowWindow( SW_HIDE );
        GetDlgItem( IDCANCEL )->ShowWindow( SW_SHOW );
        break;
    default:
        break;
    }
    UpdateData( FALSE );
    return 0;
}

//----------------------------------------------------------------------
//! \brief Handler for File Transfer Progress event.
//! \details Updates the progress bar and mTransferStatus text with the
//!     progress.
//! \return 0, always
//----------------------------------------------------------------------
afx_msg LRESULT CFileTransferProgressDlg::OnFileTransferProgress( WPARAM, LPARAM )
{
    //update progress bar
    mProgressBar.SetPos( mCom.mFileTransferBytesDone );

    //update label
    mTransferStatus.Format
        (
        _T("Sent %u/%u bytes"),
        mCom.mFileTransferBytesDone,
        mCom.mFileTransferSize
        );
    UpdateData( FALSE );

    return 0;

}    /* OnFileTransferStateChange() */

//----------------------------------------------------------------------
//! \brief Click handler for the Stop button
//! \details Tells FmiApplicationLayer to stop the transfer
//----------------------------------------------------------------------
void CFileTransferProgressDlg::OnBnClickedStop()
{
    mStopButtonClicked = TRUE;
    mCom.stopFileTransfer();
}    /* OnBnClickedStop() */

