/*********************************************************************
*
*   HEADER NAME:
*       EventId.h
*
*   Copyright 2009-2016 by Garmin Ltd. or its subsidiaries.
*---------------------------------------------------------------------
* $NoKeywords$
*********************************************************************/
#ifndef EventId_H
#define EventId_H

#include "stdafx.h"

#include "fmi.h"
#include "MessageId.h"

//! Events generated by the FMI PC Application
enum EventId
{
    EVENT_MAIN_DLG_INIT,
    EVENT_COMM_TIMEOUT,
    EVENT_COMM_ERROR_DLG_CLOSED,
    EVENT_ESN_RECEIVED,
    EVENT_PVT_RECEIVED,
    EVENT_LOG_PACKET,
    EVENT_PRODUCT_ID_RECEIVED,
    EVENT_PROTOCOLS_RECEIVED,
    EVENT_LOG_VIEWER_CLOSED,
    EVENT_STOP_LIST_DLG_CLOSED,

#if FMI_SUPPORT_A602
    EVENT_FMI_TXT_MSG_ACK,
#endif

#if FMI_SUPPORT_A603
    EVENT_FMI_TXT_MSG_FROM_CLIENT,
    EVENT_FMI_ETA_RECEIVED,
    EVENT_FMI_STOP_STATUS_CHANGED,
#endif

#if FMI_SUPPORT_A604
    EVENT_FMI_MSG_THROTTLE_ERROR,
    EVENT_FMI_TXT_MSG_STATUS_RECEIVED,
    EVENT_FMI_UI_UPDATE_ERROR,
    EVENT_FMI_PING_RECEIVED,
    EVENT_FMI_PING_RESPONSE_RECEIVED,
    EVENT_FMI_FILE_TRANSFER_PROGRESS,
    EVENT_FMI_FILE_TRANSFER_STATE_CHANGED,
    EVENT_FMI_GPI_FILE_INFO_RECEIVED,
    EVENT_FMI_CANNED_RESP_LIST_RCPT_ERROR,
    EVENT_FMI_A604_TXT_MSG_ERROR,
    EVENT_FMI_CANNED_MSG_LIST_CHANGED,
    EVENT_FMI_CANNED_RESP_LIST_CHANGED,
    EVENT_FMI_DRIVER_ID_CHANGED,
    EVENT_FMI_DRIVER_STATUS_CHANGED,
    EVENT_FMI_DRIVER_STATUS_LIST_DELETE_FAILED,
    EVENT_FMI_DRIVER_STATUS_LIST_CHANGED,
    EVENT_FMI_DISABLED,
    EVENT_FMI_CANNED_MESSAGE_DLG_CLOSED,
    EVENT_FMI_CANNED_RESPONSE_DLG_CLOSED,
    EVENT_FMI_DRIVER_ID_AND_STATUS_DLG_CLOSED,
    EVENT_FMI_DISABLE_COMPLETE,
#endif

#if FMI_SUPPORT_A605
    EVENT_FMI_MSG_THROTTLE_QUERY_RESP_RECEIVED,
#endif

#if FMI_SUPPORT_A606
    EVENT_FMI_SAFE_MODE_ERROR,
#endif

#if FMI_SUPPORT_A607
    EVENT_FMI_WAYPOINT_DLG_CLOSED,
    EVENT_FMI_WAYPOINT_LIST_CHANGED,
    EVENT_FMI_CATEGORY_LIST_CHANGED,
    EVENT_FMI_DELETE_TEXT_MESSAGE_STATUS,
#endif

#if FMI_SUPPORT_A608
    EVENT_FMI_SPEED_LIMIT_SET_RESULT,
#endif

#if FMI_SUPPORT_A610
    EVENT_FMI_AOBRD_FILE_TRANSFER_RECEIVED,
#endif

#if FMI_SUPPORT_A611
    EVENT_FMI_LONG_TEXT_MSG_ERROR,
#endif

#if FMI_SUPPORT_A612
    EVENT_FMI_FORM_DELETED,
    EVENT_FMI_FORM_POSITION_CHANGED,
    EVENT_FMI_FORM_SUBMIT_RECEIVED,
#endif

#if FMI_SUPPORT_A614
    EVENT_FMI_ROUTE_CALCULATED,
#endif

#if FMI_SUPPORT_A615
    EVENT_FMI_IFTA_FILE_TRANSFER_RECEIVED,
#endif

#if FMI_SUPPORT_A616
    EVENT_FMI_BAUD_RATE_SYNCHRONIZING,
    EVENT_FMI_BAUD_RATE_SYNCHRONIZED,
#endif

#if CDT_SUPPORT
    EVENT_CDT_PING_RECEIVED,
    EVENT_CDT_PACKET_RECEIVED,
#endif

#if FMI_SUPPORT_A621
    EVENT_FMI_FORM_SHOW_FAILED,
    EVENT_FMI_FORM_SHOW_SUCCESS,
#endif

#if FMI_SUPPORT_A622
    EVENT_FMI_DASHCAM_LOCK_RECEIPT,
    EVENT_FMI_DASHCAM_CONFIG_RECEIPT,
    EVENT_FMI_DASHCAM_GET_SETTINGS_RESPONSE,
    EVENT_FMI_DASHCAM_SET_NTFCTN_RECEIPT,
    EVENT_FMI_DASHCAM_GET_NTFCTN_SETTINGS_RESPONSE,
    EVENT_FMI_DASHCAM_NTFCTN,
    EVENT_FMI_DASHCAM_INCIDENT_FILE_NTFCTN,
#endif

#if FMI_SUPPORT_A623
    EVENT_FMI_ETA_MODE_RECEIVED,
#endif

    EVENT_ID_CNT
};

#if( FMI_SUPPORT_A602 )
//! Structure containing details for a text message ACK from the client
struct text_msg_ack_event_type
{
    MessageId            message_id;        //!< The message ID that was ACKed
    char                 ack_text[50];      //!< Text of the ack (OK, Yes, No, or a canned response)
};

//! Structure containing details for a client to server text message
struct text_msg_from_client_event_type
{
    uint32               message_id;        //!< The message ID
    time_type            origination_time;  //!< Origination time on the client
    char                 message_text[200]; //!< Message sent by the client
#if( FMI_SUPPORT_A607 )
    MessageId            link_id;           //!< ID of server-to-client message that this is a reply for
    sint32               latitude;          //!< Latitude of the device sending the message
    sint32               longitude;         //!< Longitude of the device sending the message
#endif
};
#endif

#if( FMI_SUPPORT_A604 )
//! Structure containing details for a text message status from the client
struct text_msg_status_event_type
{
    MessageId                msg_id;          //!< The message ID
    fmi_A604_message_status  message_status;  //!< The status of the text message
};
#endif

#endif
