/*********************************************************************
*
*   MODULE NAME:
*       Utility.cpp
*
* Copyright 2008-2016 by Garmin Ltd. or its subsidiaries.
*---------------------------------------------------------------------
* $NoKeywords$
*********************************************************************/
#include <time.h>

#include "Utility.h"

//----------------------------------------------------------------------
//! \brief Cumulative number of days in prior months in a non-leap year
//----------------------------------------------------------------------
static uint32 const sYearDays[ 12 ]  = { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 };

//----------------------------------------------------------------------
//! \brief Get the current server time in Garmin format
//! \return The current time as a time_type
//----------------------------------------------------------------------
time_type get_current_garmin_time()
{
    time_t   aCurrentTime;
    tm       aCurrentTm;

    time( &aCurrentTime );
    //convert to nice struct that separates into years since 1900, days, hours, minutes...
#ifdef __linux__
    gmtime_r( &aCurrentTime, &aCurrentTm);
#elif defined _WIN32
    gmtime_s( &aCurrentTm, &aCurrentTime);
#endif

    return get_garmin_time( aCurrentTm );
}

//----------------------------------------------------------------------
//! \brief Get the current server time in Garmin format
//! \return The current time as a time_type
//----------------------------------------------------------------------
time_type get_garmin_time( tm aTimeTm )
{
    //Garmin time counts seconds from Dec 31, 1989
    //First, find seconds from Jan 1 1990 to aCurrentTm, excluding leap days
    time_type seconds = ( ( aTimeTm.tm_year - 90 ) * SECONDS_PER_DAY * DAYS_IN_1_YEAR )
                      + ( aTimeTm.tm_yday          * SECONDS_PER_DAY    )
                      + ( aTimeTm.tm_hour          * SECONDS_PER_HOUR   )
                      + ( aTimeTm.tm_min           * SECONDS_PER_MINUTE )
                      + ( aTimeTm.tm_sec                                );

    // Then add seconds from Dec 31 1989 to Jan 1 1990
    seconds += SECONDS_PER_DAY;

    //adjust for leap days
    // 88 last leap year prior to base year
    // (subtract 88 since time_t base year is 1900)
    int leapDays = ( aTimeTm.tm_year - 88 ) / 4;

    // if in a leap year and on or before Feb 28
    //   -> leap day has not happened this year
    if( ( aTimeTm.tm_year % 4 == 0 ) &&
        ( (uint32) aTimeTm.tm_yday <= sYearDays[2] ) )
    {
        leapDays--;
    }
    seconds += leapDays * SECONDS_PER_DAY;
    return seconds;
}