/*********************************************************************
*
*   MODULE NAME:
*       SensorList.cpp
*
*   Copyright 2014 by Garmin Ltd. or its subsidiaries.
*---------------------------------------------------------------------
* $NoKeywords$
*********************************************************************/

#define SAVE_SENSORS  "sensors.dat"

#include "stdafx.h"
#include "fmi.h"
#include "SensorList.h"

#if( FMI_SUPPORT_A617 )

//! File name of the saved sensors
//! \since Protocol A617

//----------------------------------------------------------------------
//! \brief Constructor
//----------------------------------------------------------------------

SensorList::SensorList():
    mSensors( SAVE_SENSORS )
{
    std::map<uint32,SensorListItem>::const_iterator it = mSensors.begin();
    for( ; it != mSensors.end(); it++ )
        {
        if( mSensors.get( it->first ).isValid() )
            {
            mSensorIndexInList[ mSensors.get( it->first ).getDisplayIndex() ] = mSensors.get( it->first ).getUniqueId();
            }
        }
}

//----------------------------------------------------------------------
//! \brief Add a new item to the Sensor map and index map
//----------------------------------------------------------------------
void SensorList::add( SensorListItem item )
{
std::map<uint8,uint32>::const_iterator it = mSensorIndexInList.find( item.getDisplayIndex() );
if( ( it != mSensorIndexInList.end() ) && it->second != item.getUniqueId() && item.isValid() )
        {
        std::map<uint8,uint32>::const_iterator curr;
        std::map<uint8,uint32>::const_iterator end;
        //should the loop traverse up?
        bool up = true;
        //handle rearranging all existing elements
        if( mSensors.contains( item.getUniqueId() ) )
            {
            if( mSensors.get( item.getUniqueId() ).getDisplayIndex() > item.getDisplayIndex() )
                {
                    curr = mSensorIndexInList.find( mSensors.get( item.getUniqueId() ).getDisplayIndex() );
                    curr--;
                    end = mSensorIndexInList.find( item.getDisplayIndex() );
                }
            else if( mSensors.get( item.getUniqueId() ).getDisplayIndex() < item.getDisplayIndex() )
                {
                    curr = mSensorIndexInList.find( item.getDisplayIndex() );
                    curr++;
                    end = mSensorIndexInList.find( mSensors.get( item.getUniqueId() ).getDisplayIndex() );
                    up = false;
                }
            }
        //add new element
        else
            {
            curr = mSensorIndexInList.end();
            curr--;
            end = mSensorIndexInList.find( item.getDisplayIndex() );
            }
            uint8 di;
            while( curr != end )
                {
                di = mSensors.get( curr->second ).getDisplayIndex();
                if( up )
                    ++di;
                else
                    --di;
                mSensors.get( curr->second ).setDisplayIndex( di );
                mSensorIndexInList[ mSensors.get( curr->second ).getDisplayIndex() ] = mSensors.get( curr->second ).getUniqueId();
                if( up )
                    curr--;
                else
                    curr++;
                }

            di = mSensors.get( curr->second ).getDisplayIndex();
            if( up )
                ++di;
            else
                --di;
            mSensors.get( curr->second ).setDisplayIndex( di );
            mSensorIndexInList[ mSensors.get( curr->second ).getDisplayIndex() ] = mSensors.get( curr->second ).getUniqueId();
        }
    //if changing index of an existing sensor or reusing id, get rid of old index entry to avoid double entries
    if( mSensors.contains( item.getUniqueId() ) && mSensorIndexInList[ mSensors.get( item.getUniqueId() ).getDisplayIndex() ] == item.getUniqueId() )
        {
        mSensorIndexInList.erase( mSensors.get( item.getUniqueId() ).getDisplayIndex() );
        }
    mSensorIndexInList[ item.getDisplayIndex() ] = item.getUniqueId();
    mSensors.put( item );
}

#endif