/*********************************************************************
*
*   HEADER NAME:
*       SensorListItem.h
*
*   Copyright 2014 by Garmin Ltd. or its subsidiaries.
*---------------------------------------------------------------------
* $NoKeywords$
*********************************************************************/
#ifndef _SENSORLISTITEM_H
#define _SENSORLISTITEM_H

#include "ClientListItem.h"
#include "fmi.h"
#include "FmiApplicationLayer.h"

#if( FMI_SUPPORT_A617 )

//! \brief Data structure to hold details of an A617
//!     sensor that the server needs to keep.
//! \since Protocol A617
class SensorListItem : public ClientListItem
{
public:

    SensorListItem();

    typedef uint32 key_type;

    uint32 getNumUpdates() const;

    uint32 getUniqueId() const;

    uint16 getIcon() const;

    uint8 getDisplayIndex() const;

    const char* getName() const;

    uint8 getSeverity() const;

    boolean getPlaySound() const;

    boolean getRecordSensor() const;

    const char* getStatus() const;

    const char* getDescription() const;

    void setParent
        (
        FileBackedMap<SensorListItem> * aParent
        );

    virtual void readFromStream
        (
        std::istream &aStream
        );

    virtual void writeToStream
        (
        std::ofstream &aStream
        );

    void setConfig
        (
        fmi_sensor_config_request aConfig
        );

    void setUpdate
        (
        fmi_sensor_update_request mUpdate
        );

    void setCurrentName
        (
        CString aName
        );

    void setDisplayIndex
        (
        uint8 index
        );

private:
    //keep track of the number of updates for each sensor for
    //code convenience
    uint32 numUpdates;

    fmi_sensor_config_request mConfig;

    fmi_sensor_update_request mUpdate;

    //! The map that this item belongs to, or NULL if none
    FileBackedMap<SensorListItem> * mParent;

    uint32 getChangeId() const;
};

#endif

#endif