/*********************************************************************
*
*   HEADER NAME:
*       SerialPort.h
*
*   Copyright 2008-2016 by Garmin Ltd. or its subsidiaries.
*---------------------------------------------------------------------
* $NoKeywords$
*********************************************************************/
#ifndef SerialPort_H
#define SerialPort_H

class SerialPort;
class TcpIpPort;

#include <list>

#include "stdafx.h"
#include "garmin_types.h"
#include "TimerListener.h"
#include "PhysicalLayer.h"

#define TCP_PORT_NAME "TCP_PORT_6000"

class LinkLayer;

//----------------------------------------------------------------------
//! \brief Physical layer implementation for a serial port
//----------------------------------------------------------------------
class SerialPort : public PhysicalLayer, public TimerListener
{
public:
    virtual ~SerialPort();

    static SerialPort * getInstance();
    static void destroyInstance();
    static void getPortList
        (
        std::list<CString> &aList
        );

    static bool initSerialPort
        (
        const CString& aPortName
        );

    bool init
        (
        const CString& aPortName
        );

    virtual bool setBaudRate
        (
        uint32 aBaudRate
        );

    virtual uint32 getBaudRate() const;

    const CString& getLastError() const;
    virtual const CString& getPortName() const;

    virtual bool isOpen() const;

    void onTimer();

    virtual bool tx
        (
        uint8 * aData,
        uint16  aSize
        );

protected:
    static bool getPortListFromRegistry
        (
        std::list<CString> &aList
        );

    static void getPortListEnum
        (
        std::list<CString> &aList
        );

    SerialPort();

    void recordErrorText
        (
        const CString& aOperation
        );

    virtual void pumpRx();

    virtual void close();

    //! The one and only instance of this object
    static SerialPort *     sInstance;

    friend class TcpIpPort;

private:
    //! File handle for the com port that is open, or INVALID_HANDLE_VALUE
    //!     if the port is not open
    HANDLE                  mComPortHandle;

    //! If TRUE, mComPortHandle is initialized.
    BOOL                    mHandleInitialized;

    //! Display name of the serial port being used for communication
    CString                 mPortName;

    //! String containing the last communication error.
    //! \note This is currently only used for errors reported by
    //!     SerialPort::init()
    CString                 mLastErrorText;

    //! The last successfully applied baud rate
    uint32                  mBaudRate;
};

#endif
